# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring,global-statement,no-self-use

import syslog
import re


class LoggerException(Exception):
    """Used for all exceptions that are raised from pgbouncemgr.logger."""

class SyslogLogTargetException(Exception):
    """Used for all exceptions that are raised from the SyslogLogTarget."""


class Logger(list):
    def debug(self, msg):
        for log_target in self:
            log_target.debug(msg)

    def info(self, msg):
        for log_target in self:
            log_target.info(msg)

    def warning(self, msg):
        for log_target in self:
            log_target.warning(msg)

    def error(self, msg):
        for log_target in self:
            log_target.error(msg)


def format_ex(exception):
    """Takes an exception as its input and returns a message that can be used
       for logging purposes: single line, multiple spaces collapsed."""
    error = str(exception).strip()
    error = re.sub(r' +', ' ', error)
    error = re.sub(r'\r?\n+', ' ', error)
    error = re.sub(r'\s{2,}', ' / ', error)
    name = type(exception).__name__
    return "%s: %s" % (name, error)


class MemoryLogTarget(list):
    """MemoryTarget is used to collect log messages in memory."""
    def debug(self, msg):
        self.append(["DEBUG", msg])

    def info(self, msg):
        self.append(["INFO", msg])

    def warning(self, msg):
        self.append(["WARNING", msg])

    def error(self, msg):
        self.append(["ERROR", msg])


class ConsoleLogTarget():
    """ConsoleTarget is used to send log messages to the console."""
    def __init__(self, verbose, debug):
        self.verbose_enabled = verbose or debug
        self.debug_enabled = debug

    def debug(self, msg):
        if self.debug_enabled:
            print("[DEBUG] %s" % msg)

    def info(self, msg):
        if self.verbose_enabled:
            print("[INFO] %s" % msg)

    def warning(self, msg):
        if self.verbose_enabled:
            print("[WARNING] %s" % msg)

    def error(self, msg):
        if self.verbose_enabled:
            print("[ERROR] %s" % msg)


class SyslogLogTarget():
    """Syslogtarget is used to send log messages to syslog."""
    def __init__(self, ident, facility):
        facility = self._resolve_facility(facility)
        syslog.openlog(ident=ident, facility=facility)

    def debug(self, msg):
        syslog.syslog(syslog.LOG_DEBUG, msg)

    def info(self, msg):
        syslog.syslog(syslog.LOG_INFO, msg)

    def warning(self, msg):
        syslog.syslog(syslog.LOG_WARNING, msg)

    def error(self, msg):
        syslog.syslog(syslog.LOG_ERR, msg)

    def _resolve_facility(self, facility):
        """Resolve the provided facility. When it is numeric, then use the
           number. Otherwise try to look it up in the syslog lib by name."""
        try:
            return int(facility)
        except ValueError:
            pass

        try:
            return int(getattr(syslog, str.upper(facility)))
        except (AttributeError, ValueError):
            raise SyslogLogTargetException(
                "Invalid syslog facility provided (facility=%s)" %
                (facility if facility is None else repr(facility)))
