# -*- coding: utf-8 -*-
"""The manager implements the main process that keeps track of changes in the
   PostgreSQL cluster and that reconfigures pgbouncer when needed."""

from argparse import ArgumentParser
from pgbouncemgr.logger import Logger, ConsoleLogTarget, SyslogLogTarget
from pgbouncemgr.config import Config
from pgbouncemgr.state import State
from pgbouncemgr.state_store import StateStore


DEFAULT_CONFIG = "/etc/pgbouncer/pgbouncemgr.yaml"
DEFAULT_LOG_FACILITY = "LOG_LOCAL1"


class Manager():
    def __init__(self, argv=None):
        args = _parse_arguments(argv)
        self.config = Config(args.config)
        self._create_logger(args)
        self._create_state()

    def _create_logger(self, args):
        self.log = Logger()
        self.log.append(ConsoleLogTarget(args.verbose, args.debug))
        if args.log_facility.lower() != 'none':
            self.log.append(SyslogLogTarget("pgbouncemgr", args.log_facility))

    def _create_state(self):
        self.state = State.fromConfig(self.config, self.log)
        self.state_store = StateStore(self.config.state_file, self.state)
        self.state_store.load()

    def start(self):
        self.log.info("Not yet!")
        self.log.debug("Work in progres...")
        self.log.warning("Beware!")
        self.log.error("I will crash now")


def _parse_arguments(args):
    parser = ArgumentParser(description="pgbouncemgr")
    parser.add_argument(
        "-v", "--verbose",
        default=False, action="store_true",
        help="enable verbose console output (default: disabled)")
    parser.add_argument(
        "-d", "--debug",
        default=False, action="store_true",
        help="enable debugging console output (default: disabled)")
    parser.add_argument(
        "-f", "--log-facility",
        default=DEFAULT_LOG_FACILITY,
        help="syslog facility to use or 'none' to disable syslog logging " +
             "(default: %s)" % DEFAULT_LOG_FACILITY)
    parser.add_argument(
        "--config",
        default=DEFAULT_CONFIG,
        help="config file (default: %s)" % DEFAULT_CONFIG)
    return parser.parse_args(args)


if __name__ == "__main__":
    Manager(None).start()
